import pygame, os, string

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

titlefont = os.path.join(Config.data, 'Vera.ttf')
codefont = os.path.join(Config.data, 'VeraMono.ttf')

def fit_text((w, h), text): #, fontsize=64):
    # start off trying for best fit
    fontsize = h / len(text)
    while 1:
        font = get_font(codefont, fontsize)
        tw = th = 0
        for line in text:
            width, height = font.size(line)
            tw = max(width, tw)
            th += height
        if tw < w and th < h:
            return tw, th, fontsize
        fontsize = int(fontsize * .9)

class CodeFilePage(Page):
    def __init__(self, title, file, bgcolour=Config.code_bgcolour,
            fgcolour=Config.code_fgcolour):
        self.pagetext = [line.rstrip() for line in open(file)]
        self.title = title
        self.fgcolour = fgcolour
        self.bgcolour = bgcolour

    def init(self, screen):
        sw = screen.get_rect().width
        fontsize = 64
        while 1:
            font = get_font(titlefont, fontsize)
            if font.size(self.title)[0] < sw:
                break
            fontsize = int(fontsize * .75)
        self.title = font.render(self.title, 1, Config.title_fgcolour)
        th = self.title.get_rect().height

        w, h = screen.get_rect().size
        h -= th
        w, h, fontsize = fit_text((w, h), self.pagetext)
        self.pageblock = pygame.Surface((w, h))
        self.pageblock.fill(self.bgcolour)
        font = get_font(codefont, fontsize)
        y = 0
        for line in self.pagetext:
            line = font.render(line, 1, self.fgcolour)
            self.pageblock.blit(line, (0, y))
            y += line.get_rect().height

    def render(self, screen, deltat):
        screen.fill(Config.page_bgcolour)

        sw, sh = screen.get_rect().size

        tw, th = self.title.get_rect().size
        screen.blit(self.title, (sw/2 - tw/2, 0))

        bw, bh = self.pageblock.get_size()
        l = sw/2 - bw/2
        t = th + (sh-th)/2 - bh/2
        screen.blit(self.pageblock, (l, t))




import pygame, os
from pygame.locals import *

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

FONT_SIZE = 64

font = get_font(os.path.join(Config.data, 'Vera.ttf'), FONT_SIZE)
fontIt = get_font(os.path.join(Config.data, 'VeraIt.ttf'), FONT_SIZE)
fontBd = get_font(os.path.join(Config.data, 'VeraBd.ttf'), FONT_SIZE)
fontBI = get_font(os.path.join(Config.data, 'VeraBI.ttf'), FONT_SIZE)
fontmono = get_font(os.path.join(Config.data, 'VeraMono.ttf'),
    FONT_SIZE)

LINE_SPACE = FONT_SIZE/2

class BlankLine:
    pass

class TextPage(Page):
    '''Displays some text centered on screen.

    The first line is always bolded.

    Obeys newlines.

    Lines starting withj '-' (dash) are initially hidden and are exposed
    or hidden with the left/right arrow keys. Modified arrow keys are
    ignored here, so will change pages.
    '''
    def __init__(self, text):
        self.width = 0
        self.height = 0
        self.lines = []
        self.hidden_lines = []
        for line in text.strip().splitlines():
            if line.startswith('-'):
                line = line[1:]
                hidden = True
            else:
                hidden = False

            if not line:
                # blank line, don't bother rendering
                self.lines.append(BlankLine)
                self.height += FONT_SIZE + LINE_SPACE
                continue

            if line.startswith('='):
                line = fontBd.render(line[1:], 1, Config.title_fgcolour)
            else:
                line = font.render(line, 1, Config.text_fgcolour)
            if hidden:
                self.hidden_lines.append(line)
            else:
                self.lines.append(line)
            self.width = max(self.width, line.get_rect().width)
            self.height += FONT_SIZE + LINE_SPACE

        # so we can pop() lines off
        self.hidden_lines.reverse()
        self.popped_lines = []
        self.need_render = True

    def handleEvent(self, event):
        if event.type == KEYDOWN:
            if event.mod & (KMOD_SHIFT|KMOD_CTRL|KMOD_META|KMOD_ALT):
                return event
            if event.key == K_RIGHT:
                if not self.hidden_lines:
                    return event
                line = self.hidden_lines.pop()
                self.popped_lines.append(line)
                self.lines.append(line)
                self.need_render = True
                return None
            elif event.key == K_LEFT:
                if not self.popped_lines:
                    return event
                line = self.popped_lines.pop()
                self.lines.pop()
                self.hidden_lines.append(line)
                self.need_render = True
                return None
        return event

    def render(self, screen, deltat):
        screen.fill(Config.page_bgcolour)
        w = screen.get_rect().width / 2
        h = screen.get_rect().height / 2
        hoffs = h - self.height/2
        for n, line in enumerate(self.lines):
            if line is BlankLine:
                continue
            r = line.get_rect()
            lw = r.w/2
            screen.blit(line, (w-lw, hoffs + n * FONT_SIZE * 1.5))
        self.need_render = False





class Page:
    def init(self, screen):
        pass
    def activate(self, screen):
        pass
    def handleEvent(self, event):
        return event
    def render(self, screen, deltat):
        pass




import os

path = os.path.abspath(os.path.join(os.getcwd(), __file__))

class Config:
    page_bgcolour = (0, 0, 0)
    text_fgcolour = (255, 255, 255)
    title_fgcolour = (255, 55, 55)

    code_bgcolour = (50, 50, 50)
    code_fgcolour = (0, 255, 0)

    cursor_fgcolour = (200, 200, 255)

    data = os.path.join(os.path.split(os.path.split(path)[0])[0], 'data')



import pygame

from bruce.interp import InterpreterPage, MyInterpreter, LINE_HEIGHT

class ScratchPadInterpreterPage(InterpreterPage):
    '''Run a Python interpreter with an attached PyGame scratch pad.
    '''
    def init(self, screen):
        InterpreterPage.init(self, screen)

        self.scratch = pygame.Surface((256, 256))
        self.scratch.fill((255, 255, 255))

        self.interpreter = MyInterpreter({'scratch': self.scratch,
            'pygame': pygame})

    def render(self, screen, deltat):
        InterpreterPage.render(self, screen, deltat)

        # blit the scratch pad
        x = screen.get_rect().width - 256
        if len(self.rendered_lines)*LINE_HEIGHT > 300:
            screen.blit(self.scratch, (x -10, LINE_HEIGHT + 10))
        else:
            y = screen.get_rect().height - 256
            screen.blit(self.scratch, (x -10, y-10))




import sys, os

import pygame
from pygame.locals import *

pygame.init()
pygame.font.init()

from bruce.config import Config

def main():
    location = sys.argv[1]
    if location.endswith('/'): location = location[:-1]
    presentation = __import__(location)
    if os.path.isdir(location):
        os.chdir(location)

    # init screen
    fullscreen = '-win' in sys.argv
    viewport = (1024, 768)
    if fullscreen:
        screen_flags = HWSURFACE | DOUBLEBUF
    else:
        screen_flags = HWSURFACE | FULLSCREEN | DOUBLEBUF
    screen = pygame.display.set_mode(viewport, screen_flags)
    pygame.mouse.set_visible(False)

    # get the pages
    pages = presentation.pages()
    for page in pages:
        page.init(screen)
    page_position = 0
    oldpos = None

    clock = pygame.time.Clock()
    while 1:
        deltat = clock.tick(30)

        # handle events -- NOTE THAT THE PAGE CHANGES AFTER WE
        # HANDLE ALL EVENTS
        for event in pygame.event.get():
            if event.type == NOEVENT:
                continue
            if event.type == QUIT:
                return

            # see if the page will handle the event
            event = page.handleEvent(event)
            if event is None: continue

            if event.type == KEYDOWN:
                if event.key == K_PRINT:
                    pygame.image.save(self.screen, '/tmp/screenshot.bmp')
                elif event.key == K_ESCAPE:
                    return
                elif event.key == K_d and event.mod & KMOD_CTRL:
                    # == "next page" when in an interpreter
                    page_position += 1
                    if page_position >= len(pages):
                        page_position = len(pages)-1
                elif event.key == K_RIGHT:
                    page_position += 1
                    if page_position >= len(pages):
                        page_position = len(pages)-1
                elif event.key == K_LEFT:
                    page_position -= 1
                    if page_position < 0: page_position = 0
                elif event.key == K_DOWN:
                    page_position += 5
                    if page_position >= len(pages):
                        page_position = len(pages)-1
                elif event.key == K_UP:
                    page_position -= 5
                    if page_position < 0: page_position = 0
                elif event.key == K_END:
                    page_position = len(pages)-1
                elif event.key == K_HOME:
                    page_position = 0


        page = pages[page_position]

        if oldpos != page_position:
            page.activate(screen)
            oldpos = page_position

        # render
        page.render(screen, deltat)
        pygame.display.flip()




import pygame, os

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

font = get_font(os.path.join(Config.data, 'Vera.ttf'), 64)

BORDER = 64

# TODO use PIL for scaling down - it's so much better at it

class ImagePage(Page):
    def __init__(self, title, image):
        self.title = font.render(title, 1, Config.title_fgcolour)

        # get image
        self.image = pygame.image.load(image).convert()

    def init(self, screen):
        # resize
        w = screen.get_rect().width - BORDER*2
        h = screen.get_rect().height - BORDER*2
        imw, imh = self.image.get_rect().size
        if imw > w or imh > h:
            if imw > imh:
                ratio = float(w) / imw
                imw = w
                imh *= ratio
            else:
                ratio = float(h) / imh
                imh = h
                imw *= ratio
        self.image = pygame.transform.scale(self.image, (int(imw), int(imh)))

    def render(self, screen, deltat):
        screen.fill(Config.page_bgcolour)

        w = screen.get_rect().width
        tw = self.title.get_rect().width
        screen.blit(self.title, (w/2-tw/2, 0))

        # border
        w -= BORDER*2
        h = screen.get_rect().height

        imw, imh = self.image.get_rect().size
        screen.blit(self.image, (BORDER + w/2 - imw/2, BORDER + h/2 - imh/2))




from bruce.text import TextPage
from bruce.interp import InterpreterPage
from bruce.scratch_interp import ScratchPadInterpreterPage
from bruce.codefile import CodeFilePage
from bruce.image import ImagePage

opening_pages = '''
=Presentations In Python

Using Bruce
the Presentation Tool
---
=How do you make
=a presentation?

Edit some Python code
-Add text, image & code pages
-Create your own page types
-World domination is yours!
'''

def pages():
    l = []
    # some intro
    for page in opening_pages.split('---'):
        l.append(TextPage(page))

    l.append(InterpreterPage('Zen of Python', 'import this'))

    l.append(ScratchPadInterpreterPage('Draw to the Screen', '''
car = pygame.image.load('car.png')
car
scratch.blit(car, (100, 100))
rotated = pygame.transform.rotate(car, 45)
scratch.blit(rotated, (150, 150))
'''))

    l.append(CodeFilePage("Sample Presentation", 'example.py'))

    # examples
    l.extend([
        ImagePage("Sample Image", 'data/sample.jpg'),
        ImagePage("Sample Image", 'data/sample.jpg'),
    ])
    l.append(TextPage('''=Questions?

www.pygame.org
pyopengl.sf.net
home.gna.org/oomadness
www.imitationpickles.org/pgu
'''))
    return l




import pygame, sys, pprint, sets, code, StringIO, time, os
from pygame.locals import *

from bruce.config import Config
from bruce.page import Page
from bruce.fonts import get_font

FONT_HEIGHT = 24
LINE_HEIGHT = 26
fontBd = get_font(os.path.join(Config.data, 'VeraBd.ttf'), FONT_HEIGHT)
fontmono = get_font(os.path.join(Config.data, 'VeraMono.ttf'),
    FONT_HEIGHT)
em = fontmono.size('M')[0]

class MyInterpreter(code.InteractiveInterpreter):
    error = None
    def write(self, error):
        self.error = error

shift_keys = sets.Set((K_RSHIFT, K_LSHIFT))

class InterpreterPage(Page):
    '''Run a Python interpreter.

    Backspace deletes text.

    The optional "text" may be used to "auto-type" into the interpreter.
    If activated by control-I any further keystrokes will be taken from
    this text until the end of a line. Hitting return will then invoke
    the line as per usual and move the auto-typer on to the next line.
    Backspace is not handled. Shift-return will complete the current line
    and invoke it.

    The up and down keys will search through the input history.
    '''
    def __init__(self, title='', text='', sysver=False):
        cursor = pygame.Surface((10, 2))
        pygame.draw.rect(cursor, Config.cursor_fgcolour, (0, 0, 10, 2))
        self.cursor = cursor
        if title:
            self.title = fontBd.render(title, 1, Config.title_fgcolour)
        else:
            self.title = None

        # figure the text to render and make it poppable
        self.auto_lines = [list(line)
            for line in text.strip().splitlines()]
        self.auto_lines.reverse()
        [line.reverse() for line in self.auto_lines]

        # actual rendered lines to draw to the screen
        self.rendered_lines = []

        # insert system version?
        if sysver:
            for line in sys.version.splitlines():
                self.rendered_lines.append(fontmono.render(line, 1,
                    Config.text_fgcolour))

        # current typed line
        self.current_line = ['>', '>', '>', ' ']
        line = ''.join(self.current_line)
        self.rendered_lines.append(fontmono.render(line, 1,
                    Config.text_fgcolour))

        # input from the user which doesn't quite form complete
        # interpreter input
        self.captured_input = []

        self.history = []
        self.hist_num = 0

        self.auto_mode = False
        self.auto_line = []

        self.blink_time = 0
        self.blink_state = False

    def init(self, screen):
        self.interpreter = MyInterpreter({'pygame': pygame})
        sh = screen.get_rect().height
        if self.title:
            sh -= self.title.get_rect().height
        self.max_rendered_lines = (sh - LINE_HEIGHT) / LINE_HEIGHT
        sw = screen.get_rect().width
        self.columns = sw / em
        if sw % em: self.columns -= 1

    def handleEvent(self, event):
        if event.type == KEYDOWN:
            if event.key == K_BACKSPACE:
                if not self.auto_mode and len(self.current_line) > 4:
                    self.current_line.pop()
                    self.render_current_line()
                return None
            if event.key == K_RETURN:
                if event.mod & KMOD_SHIFT and self.auto_mode:
                    while self.auto_line:
                        glyph = self.auto_line.pop()
                        self.current_line.append(glyph)

                if self.auto_mode and self.auto_line:
                    # don't honor the return key yet
                    return None

                # evaluate the input
                input = ''.join(self.current_line[4:])

                # ignore key-up events (typically the next
                # event in the queue)
                if (input.startswith('pygame.event.wait()') or
                        input.startswith('pygame.event.poll()')):
                    pygame.event.set_blocked(KEYUP)

                # run the interpreter
                self.captured_input.append(input)
                old_stdout = sys.stdout
                output = sys.stdout = StringIO.StringIO()
                input = '\n'.join(self.captured_input)
                more = self.interpreter.runsource(input)
                sys.stdout = old_stdout

                if not more:
                    self.captured_input = []

                # handle the output
                output.seek(0)
                out = []
                for line in output.readlines():
                    out.append(line.strip())
                if self.interpreter.error:
                    out.append(self.interpreter.error.strip())
                    self.interpreter.error = None

                display_out = []
                for line in out:
                    for n in range(len(line)/self.columns + 1):
                        text = line[n*self.columns:(n+1)*self.columns]
                        r = fontmono.render(text, 1, Config.text_fgcolour)
                        self.rendered_lines.append(r)

                if (input.startswith('pygame.event.wait()') or
                        input.startswith('pygame.event.poll()')):
                    pygame.event.set_allowed(KEYUP)

                # new line
                if input:
                    self.history.append(self.current_line)
                if more:
                    self.current_line = ['.', '.', '.', ' ']
                else:
                    self.current_line = ['>', '>', '>', ' ']

                # possibly remove an extraneous line?
                if len(self.rendered_lines) > self.max_rendered_lines:
                    diff = len(self.rendered_lines) - self.max_rendered_lines
                    self.rendered_lines = self.rendered_lines[diff:]

                # render the last line
                line = ''.join(self.current_line)
                r = fontmono.render(line, 1, Config.text_fgcolour)
                self.rendered_lines.append(r)

                self.hist_num = 0
                if self.auto_mode:
                    if not self.auto_lines:
                        self.auto_mode = False
                    else:
                        self.auto_line = self.auto_lines.pop()
                return None
            if not event.mod:
                if event.key == K_UP and self.history:
                    if not self.hist_num:
                        self.history.append(list(self.current_line))
                        self.hist_num = -1
                    if abs(self.hist_num) < len(self.history):
                        self.hist_num -= 1
                    self.current_line[:] = self.history[self.hist_num]
                    self.render_current_line()
                    return None
                if event.key == K_DOWN and self.history:
                    if self.hist_num < -1:
                        self.hist_num += 1
                    self.current_line[:] = self.history[self.hist_num]
                    self.render_current_line()
                    return None
            if event.key == K_i and event.mod & KMOD_CTRL:
                self.auto_mode = not self.auto_mode
                if self.auto_mode:
                    if self.auto_lines:
                        self.auto_line = self.auto_lines.pop()
                    else:
                        self.auto_mode = False
                self.current_line[:] = ['>', '>', '>', ' ']
                return None
            if event.key == K_d and event.mod & KMOD_CTRL:
                # control-D quits interpreter
                return event
            if event.key in shift_keys:
                # we use this as a modifier
                return None
            if event.mod & (KMOD_CTRL|KMOD_META|KMOD_ALT):
                return event
            if event.key == K_ESCAPE:
                # for some reason this key has a unicode representation
                return event

            glyph = getattr(event, 'unicode', None)

            # is this a useful interpreter key?
            if not glyph or ord(glyph) > 256:
                return event

            # use auto key or user key?
            if self.auto_mode:
                if self.auto_line:
                    glyph = self.auto_line.pop()
                else:
                    return None

            self.current_line.append(glyph)
            self.render_current_line()
            return None

        return event

    def render_current_line(self):
        line = ''.join(self.current_line)
        self.rendered_lines[-1] = fontmono.render(line, 1, Config.text_fgcolour)

    def render(self, screen, deltat):
        screen.fill(Config.page_bgcolour)

        self.blink_time += deltat
        if self.blink_time > 500:
            self.blink_state = not self.blink_state
            self.blink_time = 0

        toff = 0
        if self.title:
            screen.blit(self.title, (0, 0))
            toff = self.title.get_rect().height

        for n, line in enumerate(self.rendered_lines):
            hoff = toff + n * LINE_HEIGHT
            screen.blit(line, (0, hoff))

        if not self.blink_state:
            x = line.get_rect().width
            pygame.draw.rect(screen, Config.cursor_fgcolour,
                (x, toff + n*LINE_HEIGHT + 32, 10, 2))




#! /usr/bin/env python
#
# $Id$

import os
from distutils.core import setup

# perform the setup action
from bruce import __version__
data_files = [os.path.join('data', fn) for fn in os.listdir('data')]
setup(
    name = "bruce",
    version = __version__,
    description = "Bruce the Presentation Tool",
    long_description = '''Bruce the Presentation Tool is for Python
programmers who are tired of fighting with presentation tools. In its
basic form it allows text, code or image pages and even interactive
Python sessions. It uses PyGame and is easily extensible to add new
page types.''',
    author = "Richard Jones",
    author_email = "richard@mechanicalcat.net",
    url = "http://bruce.python-hosting.com/",
    packages = ["bruce"],
    data_files = [("share/bruce/data", data_files)],
    scripts = ["bruce-play"],
    requires = ["pygame (>1.6)"],
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Topic :: Multimedia :: Graphics :: Presentation',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
    ]
)

# vim: set filetype=python ts=4 sw=4 et si
